   ! EXTERNAL SUBROUTINES 
include 'f90_kind.f90'
include 'cortai.f90'
include 'jul.f90'
include 'hf_jason_doris_freq.f90'
include 'modele_fsat_doris.f90'
include 'interpol_datemes.f90'
include 'date_cycle.f90'
include 'lecture_doris_file_init.f90'
include 'calcul_dfsat.f90'

!***********************************************************************
  !
  ! HISTORIC : November 2004
  ! ----------
  !
  ! AIM
  ! ----
  !*
  !*  Main program of the calculation of correction of the USO jason1 frequency
  !*  
  !*            
  !***********************************************************************


program modele
  use f90_kind
  ! Portable kind definition module for FORTRAN 90

  !... Implicit Declarations ... 
  implicit none
 
  ! Local scalars  measurement file
  integer(SINGLE) :: nbmes
  real(SINGLE) :: datemes_deb,datemes_fin
  integer :: ncycle
  
  ! Local scalars orbital file
  integer(SINGLE) :: nbptor
  real(SINGLE) :: dateor_deb,dateor_fin,dateor_temp
  ! Local Array
  real(SINGLE),dimension(:,:),allocatable :: tab_dfsat  

  ! Local scalars
  character*1 :: cformat
  real(SINGLE) :: duree_type
  integer(SINGLE) :: nduree,line,nformat,il

  !... External Functions ...
  external lecture_doris_file_init,date_cycle,calcul_dfsat,interpol_datemes 
    
  !... Intrinsic Functions ..
  intrinsic int

  !... Executable Statements ...

  ! cleaning file OUTPUT/jason_position.dat
  open (39,file = 'OUTPUT/jason_position.dat',status='replace')
  close (39)

  ! choice of the measurement file format 
  print 10000
100 continue
  read(*,'(a1)') cformat
  if (cformat .ne. '1' .AND. cformat .ne. '2') then
    print *,'Wrong answer, try again !'
    go to 100
  endif
  read(cformat,'(i1)') nformat

  ! to seek the begin and end dates and the number of measurements starting from the file of the measurements
  call lecture_doris_file_init(nformat,datemes_deb,datemes_fin,nbmes)
  ! if the begin date of measurement exceeds the end date of the data
  if (datemes_deb < 18998._S) then
    print 10003,datemes_deb
  endif
  if (datemes_fin > 19904._S) then
    print 10004,datemes_fin
  endif
 
  ! to seek the begin and end dates of jason cycle starting from the begin date of the measurements
  call date_cycle (datemes_deb,datemes_fin,dateor_deb,dateor_fin,ncycle)

  !to seek number of cycles to be considered : nduree

  duree_type = 9.91562865751777_S
  nduree = 0_S
  dateor_temp = dateor_deb
  do while ((datemes_fin - dateor_temp) >= 0_S) 
    nduree = nduree + 1
    dateor_temp = dateor_temp + duree_type
  enddo

 ! the orbit ground track of a cycle is given by 10 second step, which gives roughly 88310 values (date, latitude, longitude)
 ! allowance of memory for the array of dfsat values to calculate
  nbptor = nduree * 88310_S
  allocate(tab_dfsat(nbptor,3))

 ! initialization
  line = 1
      
 ! calculation of frequency correction (dfsat) for the cycle in progress
 ! determination of the number of calculated values of dfsat
  call calcul_dfsat(line,dateor_deb,dateor_fin,ncycle,nbptor,tab_dfsat)
 ! the begin date of measurements becomes the end date of the cycle so as to seek the following cycle
  datemes_deb = dateor_fin

 ! as long as the end date of measurement is higher than the end date of cycle, a new cycle is considered
 ! incrementing of the number of calculated values of dfsat
  do while (datemes_fin > dateor_fin)
    call date_cycle (datemes_deb,datemes_fin,dateor_deb,dateor_fin,ncycle)
    if (dateor_deb .eq. dateor_fin) exit
    call calcul_dfsat(line,dateor_deb,dateor_fin,ncycle,nbptor,tab_dfsat)
    datemes_deb = dateor_fin
  enddo
  line = line - 1

 ! interpolation of the dfsat at the date of measurement starting from the table of dfsat obtained previously
  write (*,10002)
  call interpol_datemes(nformat,nbmes,tab_dfsat,nbptor,line) 
  deallocate(tab_dfsat)

  print 10001

10000 format(/'Date equivalence: CNES ', &
  'Julian day 19000.0 <==> calendar date January 8, 2002 at 0 h'// &
  'Measurement File Name: ./ANNEXE/MESURE/Input'/ &
  'File type ?'/ &
  '1 <==> CDDIS'/ &
  '2 <==> DATES in TAI : year, day-of-year, seconds-from-midnight, microseconds')
10001 format(/'End of Calculation'//'Output Files (all dates in TAI):'/ &
  '--------------------------------'/ &
  './OUTPUT/jason_position.dat : date/lat/lon of reference orbit'/ &
  './OUTPUT/jason_df.orbit.dat : date/dfsat_ST/dfsat_LT along reference orbit'/ &
  './OUTPUT/jason_df.measu.dat : date/dfsat_ST/dfsat_LT at the ', &
    'measurement points of file ./ANNEXE/MESURE/Input'/&
  'AND IF CDDIS FILE TYPE:'/&
  './OUTPUT/jason_df.measu2.dat : CDDIS FILE corrected by short-term frequency')
10002 format(/'Interpolation at the ', &
    'measurement points of file ./ANNEXE/MESURE/Input')
10003 format('First date of the measurements (',f18.12,') outside the ', &
    'operation of DORIS_JASON.instr_2')
10004 format('Date of the measurements (',f18.12,') for the ', &
    ' DORIS_JASON.instr_1')
  
end program modele
